@@ -1,12 +1,14 @@
 import logging
+import subprocess
+from pathlib import Path
 
 from . import plugins
-from . import common
-from .engine import config, invert_dict, Plugin
-from .engine.event import Event, EventManager
 from .downloader import download, check_url
-from .engine.upload import UploadBase
+from .engine import invert_dict, Plugin
+from biliup.config import config
+from .engine.event import Event, EventManager
 from .uploader import upload
+from functools import reduce
 
 CHECK = 'check'
 CHECK_UPLOAD = 'check_upload'
@@ -38,33 +40,27 @@ event_manager = create_event_manager()
 
 @event_manager.register(DOWNLOAD, block='Asynchronous1')
 def process(name, url):
-    date = common.time_now(config['streamers'][name].get('title'))
     stream_info = {
         'name': name,
         'url': url,
-        'date': date
     }
     try:
-        kwargs = config['streamers'][name].copy()
+        kwargs: dict = config['streamers'][name].copy()
         kwargs.pop('url')
         suffix = kwargs.get('format')
         if suffix:
             kwargs['suffix'] = suffix
-        info = download(name, url, **kwargs)
-        stream_info = {**stream_info, **info}
+        stream_info = download(name, url, **kwargs)
     finally:
         return Event(UPLOAD, (stream_info,))
 
 
 @event_manager.register(UPLOAD, block='Asynchronous2')
 def process_upload(stream_info):
-    name, url, date = stream_info['name'], stream_info['url'], stream_info['date']
+    url = stream_info['url']
     yield Event(BE_MODIFIED, (url, 2))
     try:
-        data = {"url": url, "date": date}
-        if config['streamers'][name].get('title'):
-            data["format_title"] = date.format(title= stream_info.get('title') if stream_info.get('title') else "" )
-        upload(config.get("uploader") if config.get("uploader") else "bili_web", name, data)
+        upload(stream_info)
     finally:
         yield Event(BE_MODIFIED, args=(url, 0))
 
@@ -83,7 +79,7 @@ class KernelFunc:
     def singleton_check(self, platform):
         plugin = self.checker[platform]
         wait = config.get('checker_sleep') if config.get('checker_sleep') else 15
-        for url in check_url(plugin, secs=wait):
+        for url in check_url(plugin, self.url_status, secs=wait):
             yield Event(TO_MODIFY, args=(url,))
 
     @event_manager.register(TO_MODIFY)
@@ -95,6 +91,9 @@ class KernelFunc:
         if self.url_status[url] == 2:
             return logger.debug('正在上传稍后下载')
         name = self.inverted_index[url]
+
+        if config['streamers'].get(name, {}).get('preprocessor'):
+            preprocessor(config['streamers'].get(name, {}).get('preprocessor'))
         logger.debug(f'{name}刚刚开播，去下载')
         self.url_status[url] = 1
         return Event(DOWNLOAD, args=(name, url))
@@ -104,10 +103,9 @@ class KernelFunc:
         for title, urls in self.streamer_url.items():
             if self.free(urls):
                 yield Event(UPLOAD, args=({
-                    'name': title,
-                    'url': urls[0],
-                    'date': common.time_now(config['streamers'][title].get('title'))
-                },))
+                                              'name': title,
+                                              'url': urls[0],
+                                          },))
 
     @event_manager.register(BE_MODIFIED)
     def revise(self, url, status):
@@ -119,3 +117,20 @@ class KernelFunc:
     def free(self, list_url):
         status_num = list(map(lambda x: self.url_status.get(x), list_url))
         return not (1 in status_num or 2 in status_num)
+
+    def get_url_status(self):
+        return self.url_status
+
+
+def preprocessor(pre_processors):
+    for pre_processor in pre_processors:
+        if pre_processor.get('run'):
+            try:
+                process_output = subprocess.check_output(
+                    pre_processor['run'], shell=True,
+                    input=reduce(lambda x, y: x + str(Path(y).absolute()) + '\n', '', ''),
+                    stderr=subprocess.STDOUT, text=True)
+                logger.info(process_output.rstrip())
+            except subprocess.CalledProcessError as e:
+                logger.exception(e.output)
+                continue
